﻿using Microsoft.Win32;
using System;
using System.Collections;
using System.IO;
using System.Reflection;
using System.Threading;
using System.Windows;
using System.Windows.Forms;
using System.Windows.Input;
using System.Windows.Threading;

namespace GpdWinKeepAlive
{
  public partial class MainWindow : Window
  {
    Utility.AppSettings m_Settings = null;
    NotifyIcon m_TrayIcon = null;
    Thread m_KeepAliveThread = null;
    bool m_StopRequest = false;
    DispatcherTimer m_ClickTimer = null;
    bool m_Enabled = false;

    public MainWindow()
    {
      InitializeComponent();

      m_Version.Text = string.Format("Version {0}", Assembly.GetExecutingAssembly().GetName().Version.ToString());
      m_Settings = Utility.AppSettings.Load();
      if (m_Settings == null)
        m_Settings = new Utility.AppSettings();
      m_Autostart.IsChecked = m_Settings.Autostart;
      m_StartMinimized.IsChecked = m_Settings.StartMinimized;
      m_StartEnabled.IsChecked = m_Settings.StartEnabled;
      m_TargetFolder.Text = m_Settings.Target;
      m_Interval.SelectedIndex = (m_Settings.Interval / 5) - 1;
      m_Enabled = m_Settings.StartEnabled;
      m_EnabledToggle.IsChecked = m_Enabled;

      m_KeepAliveThread = new Thread(KeepAliveThread);
      m_KeepAliveThread.Start("D");

      m_ClickTimer = new DispatcherTimer();
      m_ClickTimer.Interval = TimeSpan.FromMilliseconds(200);
      m_ClickTimer.Tick += TrayIconClick;

      m_TrayIcon = new NotifyIcon();
      m_EnabledToggle_Click(null, null);
      m_TrayIcon.Visible = true;
      m_TrayIcon.Click += delegate (object sender, EventArgs args)
      {
        if (!m_ClickTimer.IsEnabled)
          m_ClickTimer.Start();
      };

      m_TrayIcon.DoubleClick += delegate (object sender, EventArgs args)
      {
        if (m_ClickTimer != null) {
          m_ClickTimer.Stop();
        }

        if (!this.IsVisible) {
          m_TrayIcon.Visible = false;
          this.Show();
          this.Activate();
          this.WindowState = WindowState.Normal;
        } else {
          this.Hide();
        }
      };
      m_TrayIcon.Visible = false;

      if (m_Settings.StartMinimized) {
        this.Hide();
        m_TrayIcon.Visible = true;
        m_TrayIcon.BalloonTipText = "Gpd Win Keepalive started";
        m_TrayIcon.ShowBalloonTip(1000);
      }
    }

    private void TrayIconClick(object sender, EventArgs e)
    {
      DispatcherTimer timer = sender as DispatcherTimer;
      timer.Stop();

      m_EnabledToggle.IsChecked = !m_EnabledToggle.IsChecked.Value;
      m_EnabledToggle_Click(null, null);
      m_TrayIcon.BalloonTipText = m_Enabled ? "Gpd Win Keepalive enabled" : "Gpd Win Keepalive disabled";
      m_TrayIcon.ShowBalloonTip(1000);
    } // TrayIconClick

    #region KeepAlive
    private void KeepAliveThread(object state)
    {
      while (!m_StopRequest) {
        if (!string.IsNullOrEmpty(m_Settings.Target) && m_Enabled) {
          string target = string.Format(@"{0}\gpdwinka.txt", m_Settings.Target);
          try {
            using (StreamWriter sw = new StreamWriter(target, false)) {
              sw.Write(string.Empty);
            }
          } catch (Exception) { }
        }

        int interval = m_Settings.Interval * 1000;
        int waited = 0;
        while (!m_StopRequest && waited < interval) {
          Thread.Sleep(500);
          waited += 500;
        }
      }
    } // KeepAliveThread
    #endregion

    #region Events
    private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
    {
      if (System.Windows.MessageBox.Show(this, "Close Gpd Win Keepalive?", "Confirm", MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.No) {
        e.Cancel = true;
        return;
      }

      m_StopRequest = true;
      if (m_KeepAliveThread != null) {
        m_KeepAliveThread.Join();
        m_KeepAliveThread = null;
      }

      if (m_TrayIcon != null) {
        m_TrayIcon.Dispose();
        m_TrayIcon = null;
      }
    } // Window_Closing

    private void Button_Click(object sender, RoutedEventArgs e)
    {
      FolderBrowserDialog dlg = new FolderBrowserDialog();
      DialogResult result = dlg.ShowDialog();
      if (result == System.Windows.Forms.DialogResult.OK) {
        m_TargetFolder.Text = dlg.SelectedPath;
      }
    }

    private void m_Apply_Click(object sender, RoutedEventArgs e)
    {
      m_Settings.Autostart = m_Autostart.IsChecked.HasValue && m_Autostart.IsChecked.Value;
      m_Settings.StartMinimized = m_StartMinimized.IsChecked.HasValue && m_StartMinimized.IsChecked.Value;
      m_Settings.StartEnabled = m_StartEnabled.IsChecked.HasValue && m_StartEnabled.IsChecked.Value;
      m_Settings.Interval = (m_Interval.SelectedIndex + 1) * 5;
      m_Settings.Target = m_TargetFolder.Text;
      m_Settings.Save();

      RegistryKey rk = Registry.CurrentUser.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Run", true);
      if (m_Settings.Autostart)
        rk.SetValue("Gpd Win Keepalive", Assembly.GetEntryAssembly().Location);
      else
        rk.DeleteValue("Gpd Win Keepalive", false);

      m_TrayIcon.Text = m_Enabled ? "Gpd Win Keepalive - Enabled" : "Gpd Win Keepalive";
      System.Windows.MessageBox.Show(this, "Settings saved and applied", "Confirm", MessageBoxButton.OK, MessageBoxImage.Information);
    }

    private void m_TargetFolder_MouseDoubleClick(object sender, MouseButtonEventArgs e)
    {
      Button_Click(null, null);
    }

    private void Window_StateChanged(object sender, EventArgs e)
    {
      if (this.WindowState == WindowState.Minimized) {
        m_TrayIcon.Visible = true;
        Hide();
      }
    }
    #endregion

    private void m_EnabledToggle_Click(object sender, RoutedEventArgs e)
    {
      m_Enabled = m_EnabledToggle.IsChecked.Value;

      using (Stream s = System.Windows.Application.GetResourceStream(
                  new Uri(m_Enabled ? 
                          "pack://application:,,,/GpdWinKeepAlive;component/Icons/enabled.ico" :
                          "pack://application:,,,/GpdWinKeepAlive;component/Icons/disabled.ico")).Stream) {
        m_TrayIcon.Icon = new System.Drawing.Icon(s);
      }
      m_TrayIcon.Text = m_Enabled ? "Gpd Win Keepalive - Enabled" : "Gpd Win Keepalive";
    }
  }
}
