﻿using System;
using System.IO;
using System.Text;
using System.Xml;
using System.Xml.Serialization;

namespace GpdWinKeepAlive.Utility
{
  public class AppSettings
  {
    static string m_SettingsFile = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData), "GpdWinKeepalive\\settings.xml");

    public AppSettings()
    {
      StartEnabled = true;
      Interval = 10;
      Target = @"D:\";
    }

    [XmlAttribute]
    public int Interval { get; set; }

    [XmlAttribute]
    public string Target { get; set; }

    [XmlAttribute]
    public bool StartEnabled { get; set; }

    [XmlAttribute]
    public bool Autostart { get; set; }

    [XmlAttribute]
    public bool StartMinimized { get; set; }

    public static AppSettings Load()
    {      
      try {
        using (FileStream fs = new FileStream(m_SettingsFile, FileMode.Open)) {
          XmlSerializer serializer = new XmlSerializer(typeof(AppSettings));
          AppSettings result;
          using (StreamReader sr = new StreamReader(fs, Encoding.UTF8)) {
            result = (AppSettings)serializer.Deserialize(sr);
          }
          return result;
        }
      } catch (Exception) {

      }
      return null;
    }

    public bool Save()
    {
      try {
        XmlWriterSettings xmlWriterSettings = new XmlWriterSettings
        {
          Indent = true,
          OmitXmlDeclaration = false,
          Encoding = Encoding.UTF8
        };

        string path = Path.GetDirectoryName(m_SettingsFile);
        if (!Directory.Exists(path))
          Directory.CreateDirectory(path);

        using (FileStream fs = new FileStream(m_SettingsFile, FileMode.Create)) {
          using (XmlWriter xmlWriter = XmlWriter.Create(fs, xmlWriterSettings)) {
            XmlSerializer serializer = new XmlSerializer(typeof(AppSettings));
            serializer.Serialize(xmlWriter, this);
          }
        }
      } catch (Exception) {
        return false;
      }
      return true;
    }
  }
}
